// 会议
$(function() {
	var overviewSettings = Ibos.app.g('overview');

	function createColumns(start, end, format) {
		var ret = [];
		format = format || 'HH:mm';

		for (; start < end; start++) {
			ret.push({
				name: moment().hour(start).minute(0).format(format)
			});
		}

		return ret;
	}

	var columns = createColumns(overviewSettings.startHour, overviewSettings.endHour);

	var collection = new OverView.ItemCollection();
	collection.url = Ibos.app.url('meeting/manager/meetingdata');

	var viewConfig = {
		el: '#meeting_overview',
		template: document.getElementById('tpl_meeting_overview').text,
		rangeStartTime: +moment().startOf('day').hours(overviewSettings.startHour),
		rangeEndTime: +moment().startOf('day').hours(overviewSettings.endHour),
		columns: columns,
		categories: overviewSettings.categories,
		grid: 0.5,
		unselectableText: Ibos.l("MEETING.ROOM_FORBID_BESPEAK"),
		collection: collection,
	};
	var view = new OverView.MainView(viewConfig);

	view.on('selectstop', function(e, model, data) {
		var contentTemplate = _.template(document.getElementById('tpl_meeting_apply').text),
				dialog;

		dialog = Ui.dialog({
			id: 'd_meeting_apply',
			title: false,
			init: function() {
				var dialog = this,
					$input = this.DOM.content.find('input').focus();
				$input.keydown(function(e) {
					if (e.which == 13) {
						this.DOM.content.find('form').submit();
					}
				});
			},
			width: 320,
			padding: '10px 20px 20px',
			content: contentTemplate({
				category: model.id,
				startday: moment(data.startTime).locale('zh-cn').format('YYYY-MM-DD'),
				dateStr: moment(data.startTime).locale('zh-cn').format('MM月DD日（ddd）'),
				begin: moment(data.startTime).format('HH:mm'),
				end: moment(data.endTime).format('HH:mm')
			}),
			close : function(){
				$("#meeting_overview .ai-item-overlay").hide();
			}
		});
		dialog.position(e.clientX - dialog.DOM.wrap.outerWidth() / 2, e.clientY - dialog.DOM.wrap.outerHeight());
	});

	view.on('unselect', function(evt) {
		var dialog = Ui.getDialog('d_meeting_apply');
		if (dialog &&
				evt.target != dialog.DOM.wrap[0] &&
				!$.contains(dialog.DOM.wrap[0], evt.target)) {
			dialog.close();
		}
	});

	collection.fetch();


	// 日期管理
	var dateManager = {
		dateDisplay: '.mini-date',
		currentDate: new Date(),
		template: '<strong><%= date %></strong> <div class="mini-date-body"> <p><%= weekday %></p> <p><%= yearMonth %></p> </div>',
		setDate: function(date) {
			this.currentDate = date;

			var mm = moment(date).locale('zh-cn');

			$(this.dateDisplay).html($.template(this.template, {
				date: mm.date(),
				weekday: mm.format('dddd'),
				yearMonth: mm.format('YYYY-MM')
			}));

			viewConfig.rangeStartTime = +moment(date).startOf('day').hours(overviewSettings.startHour);
			viewConfig.rangeEndTime = +moment(date).startOf('day').hours(overviewSettings.endHour);

			collection.fetch({
				data: {date: mm.format('YYYY-MM-DD')},
				reset: true
			});
		},
		setNextDate: function() {
			this.setDate(moment(this.currentDate).add(1, 'd').toDate());
		},
		setPrevDate: function() {
			this.setDate(moment(this.currentDate).subtract(1, 'd').toDate());
		}
	};

	// 日期选择
	$('#select_date').on('changeDate', function(event) {
		dateManager.setDate(event.localDate);
	}).datepicker({
		orientation: 'right'
	});

	$('#prev_date').click($.proxy(dateManager.setPrevDate, dateManager));
	$('#next_date').click($.proxy(dateManager.setNextDate, dateManager));
});
