var OverView = (function(){
	var ItemModel = Backbone.Model.extend({
		defaults: {
			title: '',
			category: '',
			status: 0,
			startTime: 0,
			endTime: 0
		}
	});

	var ItemCollection = Backbone.Collection.extend({
		model: ItemModel,

		filterByCategory: function(category) {
			return this.filter(function(model) {
				return model.get('category') == category;
			});
		},

		// 排序先依据开始时间，再凭据时间跨度
		comparator: function(model) {
			return model.get('startTime') * 1e4 + model.get('startTime') - model.get('endTime');
		},
	});

	var ItemView = Backbone.View.extend({
		className: 'ai-item',

		template: function(data){
			data.displayTime = this.formatTimeDisplay(data.startTime, data.endTime);

			return '<div class="ai-item-bg"></div>' + 
						'<div class="ai-item-inner">' +
							'<%= title %><%= user %> ' +
							'<p><%= displayTime %></p> ' +
						'</div> ';
		},

		getOption: function(prop) {
			return typeof this.options[prop] !== 'undefined' ? this.options[prop] : this[prop];
		},

		formatTimeDisplay: function(startTime, endTime) {
			moment.locale('zh-cn');
			var start = moment(startTime),
				end = moment(endTime);

			return start.format('MM月DD日 HH:mm') + ' - ' + end.format('MM月DD日 HH:mm');
		},


		// 获取指定时间相对于视图时间基点的位置
		getTimeOffset: function(time) {
			var opt = _.bind(this.getOption, this),
				rangeStartTime = opt('rangeStartTime'),
				rangeEndTime = opt('rangeEndTime'),
				rate = opt('columnWidth') / opt('columnInterval');

			if(time <= rangeStartTime) {
				return 0;
			} else if(time >= rangeEndTime) {
				return (rangeEndTime - rangeStartTime) * rate;
			} else {
				return (time - rangeStartTime) * rate;
			}
		},

		// 获取当前条目的起末位置
		getItemOffset: function() {
			var start = this.model.get('startTime'),
				end = this.model.get('endTime');

			return {
				start: this.getTimeOffset(start),
				end: this.getTimeOffset(end)
			};
		},

		updateDelta: function(leftDelta, rightDelta, silent) {
			// 计算出新的时间并更新 model
			var columnInterval = this.getOption('columnInterval'),
				leftDeltaTime = columnInterval * (leftDelta || 0),
				rightDeltaTime = columnInterval * (rightDelta || 0),
				startTime = this.model.get('startTime') + leftDeltaTime,
				endTime = this.model.get('endTime') + rightDeltaTime;

			this.updateTime(startTime, endTime, silent);
		},

		updateTime: function(startTime, endTime, silent) {
			this.model.save({
				'startTime': startTime,
				'endTime': endTime
			});

			(!silent) && this.model.trigger('updateTime', this.model);
		},

		_isOpEnable: function(op) {
			if(this.model.get(op) === false) {
				return false;
			}
			if(this.getOption(op) === false ) {
				return false;
			}
			return true;
		},

		// 获取条目移动的列数
		getDelta: function(start, end) {
			return Math.round((end - start) * 2 / this.getOption('columnWidth')) / 2;
		},

		dragGrid: 0.5,

		render: function() {
			var data = this.model.toJSON(),
				template = _.isFunction(this.template) ? 
				_.template(this.template(data)) :
				_.template(this.template);

			this.$el.attr("title", Ibos.l("MEETING.MEETING_NAME")+"： " + data.title +"\n"+ Ibos.l("MEETING.APPLICANT") +"： " + data.user + " \n"+ Ibos.l("MEETING.DATE") +"： "+ data.displayTime);
			this.$el.html(template(data));
			
			var offset = this.getItemOffset(),
				itemWidth = offset.end - offset.start;

			this.$el.css({
				'position': 'absolute',
				'left': offset.start
			})
			.addClass('ai-item-' + this.model.get('status'))
			.outerWidth(itemWidth);

			return this;
		},

		initialize: function(options) {
			this.options = options;
		}
	});

	var CategoryModel = Backbone.Model.extend({
		defaults: {
			name: ''
		}
	});

	var CategoryView = Backbone.View.extend({
		className: 'ai-category',

		ItemView: ItemView,

		getOption: function(prop) {
			return typeof this.options[prop] !== 'undefined' ? this.options[prop] : this[prop];
		},

		// 条目是否在视图可见范围内
		inRange: function(model) {
			var rangeStartTime = this.getOption('rangeStartTime'),
				rangeEndTime = this.getOption('rangeEndTime'),
				startTime = model.get('startTime'),
				endTime = model.get('endTime');

			if(startTime >= rangeEndTime || endTime <= rangeStartTime) {
				return false;
			}

			return true;
		},

		getModelsInRange: function() {
			var cateView = this;
			return _.filter(this.model.items, function(model) {
				return cateView.inRange(model);
			});
		},

		// 从 fullCalendar 抄过来的条目排列区间占位算法
		placeSegments: function(ms) {
			function isSlotSegCollision(seg1, seg2) {
				return seg1.endTime > seg2.startTime && seg1.startTime < seg2.endTime;
			}

			function computeSlotSegCollisions(seg, otherSegs, results) {
				results = results || [];

				for (var i=0; i<otherSegs.length; i++) {
					if (isSlotSegCollision(seg, otherSegs[i])) {
						results.push(otherSegs[i]);
					}
				}

				return results;
			}

			function computeForwardSlotSegs(levels) {
				var i, level;
				var j, seg;
				var k;

				for (i=0; i<levels.length; i++) {
					level = levels[i];

					for (j=0; j<level.length; j++) {
						seg = level[j];

						seg.forwardSegs = [];
						for (k=i+1; k<levels.length; k++) {
							computeSlotSegCollisions(seg, levels[k], seg.forwardSegs);
						}
					}
				}
			}

			function computeSlotSegPressures(seg) {
				var forwardSegs = seg.forwardSegs;
				var forwardPressure = 0;
				var i, forwardSeg;

				if (seg.forwardPressure === undefined) { // not already computed

					for (i=0; i<forwardSegs.length; i++) {
						forwardSeg = forwardSegs[i];

						// figure out the child's maximum forward path
						computeSlotSegPressures(forwardSeg);

						// either use the existing maximum, or use the child's forward pressure
						// plus one (for the forwardSeg itself)
						forwardPressure = Math.max(
							forwardPressure,
							1 + forwardSeg.forwardPressure
						);
					}

					seg.forwardPressure = forwardPressure;
				}
			}

			function compareSlotSegs(seg1, seg2) {
				return seg1.startTime - seg2.startTime || // earlier start time goes first
					(seg2.endTime - seg2.startTime) - (seg1.endTime - seg1.startTime) || // tie? longer-duration goes first
					(seg1.title || '').localeCompare(seg2.title); // tie? alphabetically by title
			}

			function compareForwardSlotSegs(seg1, seg2) {
				// put higher-pressure first
				return seg2.forwardPressure - seg1.forwardPressure ||
					// put segments that are closer to initial edge first (and favor ones with no coords yet)
					(seg1.backwardCoord || 0) - (seg2.backwardCoord || 0) ||
					// do normal sorting...
					compareSlotSegs(seg1, seg2);
			}

			function computeSlotSegCoords(seg, seriesBackwardPressure, seriesBackwardCoord) {
				var forwardSegs = seg.forwardSegs;
				var i;

				if (seg.forwardCoord === undefined) { // not already computed

					if (!forwardSegs.length) {

						// if there are no forward segments, this segment should butt up against the edge
						seg.forwardCoord = 1;
					}
					else {

						// sort highest pressure first
						forwardSegs.sort(compareForwardSlotSegs);

						// this segment's forwardCoord will be calculated from the backwardCoord of the
						// highest-pressure forward segment.
						computeSlotSegCoords(forwardSegs[0], seriesBackwardPressure + 1, seriesBackwardCoord);
						seg.forwardCoord = forwardSegs[0].backwardCoord;
					}

					// calculate the backwardCoord from the forwardCoord. consider the series
					seg.backwardCoord = seg.forwardCoord -
						(seg.forwardCoord - seriesBackwardCoord) / // available width for series
						(seriesBackwardPressure + 1); // # of segments in the series

					// use this segment's coordinates to computed the coordinates of the less-pressurized
					// forward segments
					for (i=0; i<forwardSegs.length; i++) {
						computeSlotSegCoords(forwardSegs[i], 0, seg.forwardCoord);
					}
				}
			}

			function buildSlotSegLevels(segs) {
				var levels = [];
				var i, seg;
				var j;

				for (i=0; i<segs.length; i++) {
					seg = segs[i];

					// go through all the levels and stop on the first level where there are no collisions
					for (j=0; j<levels.length; j++) {
						if (!computeSlotSegCollisions(seg, levels[j]).length) {
							break;
						}
					}

					(levels[j] || (levels[j] = [])).push(seg);
				}

				return levels;
			}

			function flattenSlotSegLevels(levels) {
				var segs = [];
				var i, level;
				var j;

				for (i=0; i<levels.length; i++) {
					level = levels[i];

					for (j=0; j<level.length; j++) {
						segs.push(level[j]);
					}
				}

				return segs;
			}

			function placeSlotSegs(segs) {
				var levels = buildSlotSegLevels(segs);
				var level0 = levels[0];
				var i;

				computeForwardSlotSegs(levels);

				if (level0) {

					for (i=0; i<level0.length; i++) {
						computeSlotSegPressures(level0[i]);
					}

					for (i=0; i<level0.length; i++) {
						computeSlotSegCoords(level0[i], 0, 0);
					}
				}

				return flattenSlotSegLevels(levels);
			}

			return placeSlotSegs(ms);
		},

		sortSegments: function(ms) {
			return ms;
		},

		// 根据始末时间计算出一系列条目中各条目的位置
		computeSegments: function(models) {
			var ms = _.map(models, function(m) {
				return m.toJSON();
			});

			ms = this.sortSegments(ms);
			return this.placeSegments(ms);
		}, 

		// 当同一时间区间里，有多个条目时，多个条目会以一定的比例纵向并列在视图上
		// 此方法用于获取指定条目，在其所在时间区间内的片段起末位置
		getSegment: function(model) {
			var cateView = this;
			var collection = model.collection;

			// 获取同分类下的条目
			var models = collection.filter(function(m) {
				return m.get('category') == model.get('category') &&
				(!(m.get('startTime') >= cateView.getOption('rangeEndTime') ||
				m.get('endTime') <= cateView.getOption('rangeStartTime')));
			});
			var segments = this.computeSegments(models);
			var top, bottom;

			_.each(segments, function(o) {
				if(o.id === model.id) {
					top = o.backwardCoord;
					bottom = o.forwardCoord;
					return false;
				}
			});

			return {
				top: top,
				bottom: bottom
			};
		},

		getSegmentOffset: function(segment) {
			var height = this.$el.height();
			// 下面留白 10% 空白以便增加新条目
			return {
				top: height * segment.top * 0.9, 
				bottom: height * segment.bottom * 0.9 *  (segment.bottom !==1 ? 1.1 : 1)
			};
		},

		renderOverlay: function() {
			if(!this.$overlay || !this.$overlay.length) {
				this.$overlay = $('<div class="ai-item-overlay posa"></div>').prependTo(this.$el);
			}
			return this.$overlay;
		},

		select: function(start, end) {
			start = start || 0;
			end = end || 0;

			var $overlay = this.renderOverlay().show();
			$overlay.css({
				'left': start * this.getOption('columnWidth'),
				'width': (end - start + 1) * this.getOption('columnWidth')
			});
		},

		unselect: function(evt) {
			if(this.$overlay) {
				this.$overlay.hide();
			}
			this.trigger('unselect', evt);
		},

		// 获取某一列起始位置的时间戳
		// 此处的 offset 值为列的位置
		getTimeByColumnOffset: function(offset) {
			return this.getOption('rangeStartTime') + offset * this.getOption('columnInterval');
		},

		// 根据给出宽度计算出列数
		getColumnNum: function(x) {
			var columnWidth = this.getOption('columnWidth');
			return Math.floor(x / columnWidth);
		},

		// 添加子节点
		addItem: function(model) {
			var cateView = this;
			var opt = _.bind(this.getOption, this);
			var ItemView = this.getOption('ItemView');
			var itemView = new this.ItemView({
				model: model,
				columnWidth: opt('columnWidth'),
				columnInterval: opt('columnInterval'),
				rangeStartTime: opt('rangeStartTime'),
				rangeEndTime: opt('rangeEndTime'),
				beforePreviewArea: opt('beforePreviewArea')
			});

			// 给条目视图对象添加分类视图对象的指向
			itemView.$el.appendTo(this.$el);
			itemView.render();

			// 由于同一时间区间可能有多个条目
			// segment 用于计算在这区间内每个条目视图上的位置
			var segment = cateView.getSegment(model);
			var segmentOffset = this.getSegmentOffset(segment);

			// 设置条目的高度与顶部距离
			itemView.$el.css({
				'top': segmentOffset.top
			});

			itemView.$el.outerHeight(segmentOffset.bottom - segmentOffset.top);

			return itemView.$el;
		},

		initialize: function(options) {
			var cateView = this;
			var model = this.model;
			this.options = options;

			$(window).on('mousedown', _.bind(this.unselect, this));
		},

		events: {
			'mousedown': 'mousedownEvent'
		},

		// 判断某一列是否可选中
		isSelectable: function(index) {
			var unselectable = this.model.get('unselectable');
			var unselectableWeekday = this.model.get('unselectableWeekday');
			var weekDay = moment(this.getOption('rangeStartTime')).days();


			if(unselectable === true) {
				return false;
			}

			if(unselectableWeekday && unselectableWeekday.length && 
				$.inArray(weekDay, unselectableWeekday) !== -1) {
				return false;
			}

			if(unselectable && unselectable.length) {
				return unselectable.indexOf(index) == -1;
			}
			return true;
		},

		// 根据鼠标选择的列，计算出实际上选择的列
		// 由于存在不可选择的列，所鼠标选择的列不一定是视图上实际选择的列
		getSelectRange: function(start, end) {
			var unselectable = this.model.get('unselectable');
			// 表示当前在鼠标选择区间内的不可选择列
			var unselectableInRange; 
			// 如果有配置了不可选择的列时，进入始末位置的计算
			if(unselectable && unselectable.length) {
				// 从左到右选择时
				if(start < end) {
					unselectableInRange = _.filter(unselectable, function(u) {
						return u >= start && u <= end;
					});
					// 实际上能选中的最右列就是区间内第一个不可选中列的前一列
					if(unselectableInRange.length) {
						end = Math.min.apply(null, unselectableInRange) - 1;
					}
				// 从右向左选择时
				} else {
					unselectableInRange = _.filter(unselectable, function(u) {
						return u >= end && u <= start;
					});
					if(unselectableInRange.length) {
						end = Math.max.apply(null, unselectableInRange) + 1;
					}
				}
			}
			return [Math.min(start, end), Math.max(start, end)];
		},

		mousemoveEvent: function(e) {
			var rangeStart = e.data,
				rangeEnd = this.getColumnNum(e.clientX - this.$el.offset().left);

			// 根据之前缓存 rangeStart 跟 rangeEnd 值
			// 判断区间是否发生变化，变化时才进一步处理
			if(rangeStart == this.rangeStart && rangeEnd == this.rangeEnd) {
				return true;
			}

			this.rangeStart = rangeStart;
			this.rangeEnd = rangeEnd;

			// 获取实际选择的区间
			var range = this.getSelectRange(rangeStart, rangeEnd);
			this.selectStart = range[0];
			this.selectEnd = range[1];
			this.select(this.selectStart, this.selectEnd);
		},

		mousedownEvent: function(e) {
			// 由条目节点触发的 mousedown 不触发选择区
			var $target = $(e.target);
			if($target.is('.ai-item') || $target.parents('.ai-item').length) {
				return true;
			}

			var cateView = this;
			var $window = $(window);
			var elemOffsetLeft = this.$el.offset().left;

			var rangeStart = this.getColumnNum(e.clientX - elemOffsetLeft);

			function getSelectParam() {
				return {
					start: cateView.selectStart,
					end: cateView.selectEnd,
					startTime: cateView.getTimeByColumnOffset(cateView.selectStart),
					endTime: cateView.getTimeByColumnOffset(cateView.selectEnd + 1)
				};
			}

			// 事件开始的列不允许选择时，直接返回
			if(!this.isSelectable(rangeStart)) {
				return true;
			}

			// 如果按正常的执行流，“mousedown” 触发 “select” 之后，会被冒泡到上层时触发的 “unselect” 给取消掉
			// 所以此处使用 “setTimeout” 调整执行顺序 
			setTimeout(function(){
				cateView.selectStart = rangeStart;
				cateView.selectEnd = rangeStart;

				// selectstart
				cateView.trigger('selectstart', e, cateView.model, getSelectParam());
				cateView.select(cateView.selectStart, cateView.selectEnd);
			}, 0);

			var mousemoveEvent = _.bind(this.mousemoveEvent, this);
			var mouseupEvent = function(e) {
				// selectend
				cateView.trigger('selectstop', e, cateView.model, getSelectParam());

				// 选择结束时，清除选择过程中产生的临时变量
				// @Todo: 是否放到 unselect 时清除比较合适??
				delete cateView.selectStart;
				delete cateView.selectEnd;
				delete cateView.rangeStart;
				delete cateView.rangeEnd;

				cateView.$el.off('mousemove', mousemoveEvent);
				// 解除文本选择禁止
				cateView.$el.off('mouseup', mouseupEvent).enableSelection();
			};
			
			this.$el.on('mousemove', rangeStart, mousemoveEvent);
			// 全局禁止文本选择
			this.$el.on('mouseup', mouseupEvent).disableSelection();
		},
		renderUnselectableColumns: function() {
			var cateView = this;
			var unselectableWeekday = this.model.get('unselectableWeekday');
			var unselectable = this.model.get('unselectable');
			var columnWidth = this.getOption('columnWidth');
			var weekDay = moment(this.getOption('rangeStartTime')).days();

			if(!unselectable && unselectableWeekday && unselectableWeekday.length) {
				unselectable = $.inArray(weekDay, unselectableWeekday) !== -1;
			}

			if(unselectable === true) {
				var $elem = $('<div class="ai-unselectable-col"></div>').css({
					position: 'absolute',
					top: '1px',
					left: '1px'
				})
				.html(this.getOption('unselectableText') || '')
				.prependTo(cateView.$el);

				$elem.outerHeight(cateView.$el.outerHeight() - 2);
				$elem.outerWidth(cateView.$el.outerWidth() - 2);

			} else if(unselectable && unselectable.length) {
				_.each(unselectable, function(u) {
					$('<div class="ai-unselectable-col"></div>').css({
						left: u * columnWidth,
						width: columnWidth,
						height: '100%',
						position: 'absolute',
						top: 0
					})
					.prependTo(cateView.$el);
				});
			}
		},
		
		render: function() {
			var cateView = this;

			this.$overlay = null;
			this.$el.empty()
			.css({
				'position': 'relative',
				'height': this.getOption('rowHeight'),
				'width': '100%'
			});

			// 生成条目视图
			// 按开始时间（即startTime）顺序生成视图
			_.each(this.model.items, function(model) {
				// 检查条目在视图范围内，是的时候才生成对应条目视图
				if(cateView.inRange(model)) {
					cateView.addItem(model);
				}
			});

			this.renderUnselectableColumns();

			return this;
		}
	});

	var MainView = Backbone.View.extend({
		className: 'ai-main',

		CategoryView: CategoryView,

		initialize: function(options) {
			var mainView = this;
			this.options = options;
			$.each(this.options.categories, function(index, item){
				if( item == null ){
					mainView.options.categories.splice(index, 1);
				}
			});
			// 相关事件绑定
			mainView.collection.on('add remove reset change:category updateTime', function(model) {
				mainView.renderCategories();
			});

			var position = mainView.$el.css('position');
			if(position !== 'relative' && position !== 'absolute' && position !== 'fixed') {
				mainView.$el.css('position', 'relative');
			}

			mainView.render();
		},

		createCategoryContainer: function() {
			// 生成容器
			var $container = $('<div></div>');
			var tableWidth = this.$table.outerWidth(),
				tableHeight = this.$table.outerHeight(),
				$firstCell = this.$table.find('th').eq(0),
				firstCellWidth = $firstCell.outerWidth(),
				firstCellHeight = $firstCell.outerHeight();

			$container.css({
				'position': 'absolute',
				'overflow': 'hidden',
				'left': firstCellWidth,
				'top': firstCellHeight,
				'width': tableWidth - firstCellWidth,
				'height': tableHeight - firstCellHeight
			});

			return $container;
		},

		// 获取行高
		getRowHeight: function(index) {
			index = index || 1;
			return this.$table.find('tr').eq(index).outerHeight();
		},

		// 获取列宽
		getColumnWidth: function(index) {
			index = index || 1;
			return this.$table.find('tr').eq(0).find('th,td').eq(index).outerWidth();
		},

		// 填充分类节点及条目节点
		renderCategories: function() {
			var mainView = this,
				options = this.options,
				grid = this.options.grid || 1,
				columns = options.columns,
				categories = options.categories,
				columnWidth = mainView.getColumnWidth() * grid,
				columnInterval = (options.rangeEndTime - options.rangeStartTime) / columns.length * grid,
				rowHeight = mainView.getRowHeight();

			this.$container.empty();

			// 生成各行视图
			_.each(categories, function(category, i) {
				var CategoryView = options.CategoryView || mainView.CategoryView;
				var cateModel = new CategoryModel(category),
					cateView;

				cateModel.items = mainView.collection.filterByCategory(category.id);
				cateModel.itemCollection = mainView.collection;
				
				cateView = new CategoryView(_.extend(
					_.omit(options, 'categories', 'collection', 'columns', 'el'),
					{
						model: cateModel,
						rowHeight: rowHeight,
						columnWidth: columnWidth,
						columnInterval: columnInterval,
					}
				));

				
				// 将 category 的事件发布到 main
				_.each(['selectstart, selectstop', 'unselect'], function(eventType) {
					cateView.on(eventType, function(a, b, c) {
						mainView.trigger(eventType, a, b, c);
					});
				});

				cateView.$el.appendTo(mainView.$container);
				cateView.render();
			});
		},

		render: function() {
			var mainView = this;
			var options = this.options;
			var columns = options.columns;
			var categories = options.categories;

			this.$el.empty();

			// 生成基本网格表格
			this.$table = $('<table></table>');
			var template = _.template(options.template || this.template);

			this.$table.html(template({
				columns: columns ,
				categories: categories,
			})).appendTo(this.$el);

			this.$container = this.createCategoryContainer().appendTo(this.$el);
			this.renderCategories();

			return this.$el;
		}
	});

	return {
		ItemModel: ItemModel,

		ItemCollection: ItemCollection,

		ItemView: ItemView,

		CategoryModel: CategoryModel,

		CategoryView: CategoryView,

		MainView: MainView
	};
})();
