/**
 * wfhandler.js
 * 工作流模块表单办理JS
 * IBOS
 * @author      banyan
 * @version $Id: wfhandler.js 8231 2016-09-02 09:29:13Z tanghang $
 */

$(function() {
    /**
     * 
     * @type String
     */
    var DISABLED_FEEDBACK = '1';
    /**
     * 
     * @type String
     */
    var FORCE_FEEDBACK = '2';

    var replyLock = 0;

    var WF = Ibos.l("WF");
    /**
     * 
     * @type type
     */
    var REGMAPPING = {
        'notempty': {
            'reg': /\S+/,
            'tip': U.lang('RULE.NOT_NULL')
        },
        'chinese': {
            'reg': /^[\u4E00-\u9FA5\uF900-\uFA2D]+$/,
            'tip': U.lang('RULE.CHINESE_ONLY')
        },
        'letter': {
            'reg': /[A-Za-z]+$/,
            'tip': U.lang('RULE.ENGLISH_ONLY')
        },
        'num': {
            'reg': /^([+-]?)\d*\.?\d+$/,
            'tip': U.lang('RULE.NUMERIC_ONLY')
        },
        'idcard': {
            'reg': /^[1-9]([0-9]{14}|[0-9]{16}([0-9]|[xX]))$/,
            'tip': U.lang('RULE.IDCARD_INVALID_FORMAT')
        },
        'mobile': {
            'reg': /^13[0-9]{9}|15[012356789][0-9]{8}|18[0256789][0-9]{8}|147[0-9]{8}$/,
            'tip': U.lang('RULE.MOBILE_INVALID_FORMAT')
        },
        'money': {
            'reg': /^(-)?(([1-9]{1}\d*)|([0]{1}))(\.(\d){1,2})?$/,
            'tip': U.lang('RULE.MONEY_INVALID_FORMAT')
        },
        'tel': {
            'reg': /^(([0\+]\d{2,3}-)?(0\d{2,3})-)?(\d{7,8})(-(\d{3,}))?$/,
            'tip': U.lang('RULE.PHONE_INVALID_FORMAT')
        },
        'zipcode': {
            'reg': /^\d{6}$/,
            'tip': U.lang('RULE.ZIP_INVALID_FORMAT')
        },
        'email': {
            'reg': /^\w+((-\w+)|(\.\w+))*\@[A-Za-z0-9]+((\.|-)[A-Za-z0-9]+)*\.[A-Za-z0-9]+$/,
            'tip': U.lang('RULE.EMAIL_INVALID_FORMAT')
        }
    };
    var commonUploadSettings = {
        post_params: { module: 'workflow' },
    };
    /**
     * 公共附件
     */
    Ibos.upload.attach({
        upload_url: Ibos.app.url('main/attach/upload', { uid: Ibos.app.g('uid'), type: 'workflow', flowprocess: Ibos.app.g('flowProcess'), hash: Ibos.app.g('swfHash') }),
        post_params: { module: 'workflow' },
        button_placeholder_id: 'global_upload_btn',
        custom_settings: {
            containerId: "g_file_target",
            inputId: "attachmentid",
        }
    });
    /**
     * 会签附件
     */
    Ibos.upload.attach({
        post_params: { module: 'workflow' },
        button_placeholder_id: 'fb_upload_btn',
        custom_settings: {
            containerId: "fb_file_target",
            inputId: "fbattachmentid",
        }
    });

    /**
     * 
     * @returns {Boolean}
     */
    function checkItem() {
        var checkStr = Ibos.app.g('checkItem');
        var check = true,
            $form = getForm();
        if ($.trim(checkStr) !== '') {
            var items = checkStr.split(',');
            $.each(items, function(i, item) {
                var sep = item.split('=');
                var regObj = REGMAPPING.hasOwnProperty(sep[1]) ? REGMAPPING[sep[1]] : null;
                if (regObj) {
                    var $el = $form.find("input[title='" + sep[0] + "'], textarea[title='" + sep[0] + "']");
                    if ($el.is('input') || $el.is('textarea')) {
                        var reg = new RegExp(regObj.reg);
                        if (!reg.test($el.val() || $el.html())) {
                            check = false;
                            Ui.tip(sep[0] + ':' + regObj.tip, 'danger');
                            $el.focus().blink();
                            return false;
                        }
                    }
                }
            });
        }
        return check;
    }
    /**
     * 
     * @param {boolean} dom
     * @returns object
     */
    function getForm(dom) {
        return dom ? Ibos.app.g("form").get(0) : Ibos.app.g("form");
    }

    /**
     * 
     * @param {type} isFree
     * @returns {undefined}
     */
    function showNext(isFree) {
        var turnType = isFree ? 'freenext' : 'shownext';
        var url = Ibos.app.url('workflow/handle/' + turnType, { key: Ibos.app.g('key'), topflag: Ibos.app.g('topflag') });
        Ui.ajaxDialog(url, {
            title: U.lang('CONFIRM_POST'),
            lock: true,
            ok: function() {
                try {
                    if (sncheckform()) {
                        $('#turn_next_form').submit();
                    }
                    return false;
                } catch (e) {}

                return true;
            },
            cancel: true
        });
    }

    var wfHandler = {
        'beforeSubmit': function(saveFlag) {
            var $form = getForm();
            // Todo::表单加载状态判断
            if (Ibos.app.g('feedback') !== DISABLED_FEEDBACK) {
                // Todo::如果不禁止会签，签章控件的判定
            }
            window.onbeforeunload = function() {};
            if (!checkItem()) {
                return false;
            }
            // 处理列表控件提交
            wfc && wfc.lvBeforeSubmit().richBeforeSubmit();
            // 写入保存状态
            $form.get(0).saveflag.value = saveFlag;
            return true;
        },
        'delFb': function(feedid, $dom) {
            $.ajax({
                url: Ibos.app.url('workflow/handle/delfb'),
                data: { feedid: feedid },
                type: "get",
                dataType: "json",
                success: function(data) {
                    if (data.isSuccess) {
                        $dom.fadeOut(function() {
                            $dom.remove();
                        });
                    } else {
                        Ui.tip(U.lang('DELETE_FAILED'), 'danger');
                        return false;
                    }
                }
            });
        }
    };

    Ibos.events.add({
        /**
         * 保存表单
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'save': function(param, $ctx, evt) {
            if (wfHandler.beforeSubmit('save')) {
                getForm(true).submit();
            }
            evt.preventDefault();
        },
        /**
         * 主办人转交下一步
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'turn': function(param, $ctx, evt) {
            // 强制会签检查
            if (Ibos.app.g('feedback') === FORCE_FEEDBACK) {
                if ($.trim(getForm(true).content.value) === '' && Ibos.app.g('fbSigned') == 0) {
                    Ui.tip(WF.FORCED_SIGN_OPINION, 'danger');
                    $(getForm(true).content).blink();
                    return false;
                }
            }
            if (wfHandler.beforeSubmit('turn')) {
                getForm(true).submit();
            }
            evt.preventDefault();
        },
        /**
         * 自由流程结束当前工作 
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'end': function(param, $ctx, evt) {
            Ui.confirm(WF.CONFIRM_END_FREE_PROCESS, function() {
                if (wfHandler.beforeSubmit('end')) {
                    getForm(true).submit();
                }
            });
            evt.preventDefault();
        },
        /**
         * 经办人办理完毕工作
         * @param {type} param
         * @param {type} $ctx
         * @returns {Boolean}
         */
        'finish': function(param, $ctx, evt) {
            // 强制会签检查
            if (Ibos.app.g('feedback') === FORCE_FEEDBACK) {
                if ($.trim(getForm(true).content.value) === '') {
                    Ui.tip(WF.FORCED_SIGN_OPINION, 'danger');
                    $(getForm(true).content).blink();
                    return false;
                }
            }
            Ui.confirm(WF.CONFIRM_FINISH_WORK, function() {
                if (wfHandler.beforeSubmit('finish')) {
                    getForm(true).submit('finish');
                }
            });
            evt.preventDefault();
        },
        /**
         * 取消保存工作(即第一次新建工作进来办理页面时出现)
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'cancel': function(param, $ctx) {
            Ui.confirm(WF.CONFIRM_UNSAVE_WORK, function() {
                var url = Ibos.app.url('workflow/new/cancel', { key: Ibos.app.g('key') });
                window.location.href = url;
            });
        },
        /**
         * 返回列表
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'return': function(param, $ctx) {
            window.location.href = Ibos.app.url('workflow/list/index', { op: 'list', type: 'todo', sort: 'all' });
        },
        /**
         * 预览流程图
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'preview': function(param, $ctx) {
            var paramStr = "menubar=0,toolbar=0,status=0,resizable=1,scrollbars=1,top=0, left=0, width=" + screen.availWidth + ", height=" + screen.availHeight;
            window.open(Ibos.app.url('workflow/preview/flow', { key: param.key }), 'viewFlow', paramStr);
        },
        /**
         * 加载会签意见回复
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'loadFbReply': function(param, $ctx) {
            var $target = $ctx.parents('ul .opinion-list').next();
            // 关闭回复面板
            if ($target.hasClass("focus")) {
                $target.removeClass("focus");
                $target.hide();
                // 打开回复面板
            } else {
                $target.addClass("focus");
                $target.show();
            }
            if (!$target.hasClass('loaded')) {
                var $repList = $target.find('.cmt-sub');
                $repList.empty().height(60).waiting(null, "mini");
                $.ajax({
                    url: Ibos.app.url('workflow/handle/loadreply', { feedid: param.feedid }),
                    type: "get",
                    dataType: "json",
                    success: function(data) {
                        if (data.isSuccess) {
                            var $res = $(data.data);
                            $repList.height("").stopWaiting().replaceWith($res);
                        }
                    }
                }, 'json');
                $target.addClass("loaded");
            }
        },
        /**
         * 
         * @param {type} param
         * @param {type} $ctx
         * @returns {Boolean}
         */
        'reply': function(param, $ctx) {
            if (replyLock === 1) {
                return;
            }
            var $target = $ctx.parent().prev(),
                $rpList = $ctx.parent().next();
            $ctx.button("loading");
            var content = $target.val();
            if ($.trim(content) === "") {
                $target.blink();
                $target.button("reset");
                return false;
            }
            $rpList.waiting(null, "mini");
            $.ajax({
                url: Ibos.app.url('workflow/handle/addreply', param),
                data: { content: content, formhash: Ibos.app.g('formHash') },
                type: "post",
                dataType: "json",
                success: function(data) {
                    if (data.isSuccess) {
                        var $res = $(data.data);
                        $rpList.prepend($res).stopWaiting();
                        $target.val("");
                        $ctx.button("reset");
                    } else {
                        Ui.tip(data.data, 'danger');
                        return false;
                    }
                }
            });
            // 计数器，预防狂刷回复
            replylock = 1;
            setTimeout(function() {
                replyLock = 0;
            }, 5000);
        },
        /**
         * 点评设置 输入区域内容
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'setreply': function(param, $ctx) {
            var target = $ctx.parents('ul .cmt-sub').prev().prev();
            target.focus().val(U.lang('REPLY') + ' ' + param.name + ' ： ');
        },
        /**
         * 删除会签点评
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'delreply': function(param, $ctx) {
            var $target = $ctx.parents('li .cmt-item');
            wfHandler.delFb(param.feedid, $target);
        },
        /**
         * 删除会签
         * @returns {undefined}
         */
        'delFb': function(param, $ctx) {
            Ui.confirm(WF.CONFIRM_DEL_FB, function() {
                wfHandler.delFb(param.feedid, $('#fb_' + param.feedid));
            });
        },
        /**
         * 
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'delFbAttach': function(param, $ctx) {
            Ui.confirm(U.lang('CONFIRM_DEL_ATTACH'), function() {
                var url = Ibos.app.url('workflow/handle/delfbattach');
                $.get(url, param, function(data) {
                    if (data.isSuccess) {
                        var $target = $ctx.parentsUntil('li');
                        $target.fadeOut(function() {
                            $target.remove();
                        });
                    } else {
                        Ui.tip(U.lang('DELETE_FAILED'), 'danger');
                    }
                }, 'json');
            });
        },
        /**
         * 删除公共附件
         * @param {type} param
         * @param {type} $ctx
         * @returns {undefined}
         */
        'delAttach': function(param, $ctx) {
            Ui.confirm(U.lang('CONFIRM_DEL_ATTACH'), function() {
                var url = Ibos.app.url('workflow/handle/delattach');
                $.get(url, param, function(data) {
                    if (data.isSuccess) {
                        var $target = $('#attach_' + param.aid);
                        var $attachmentid = $("#attachmentid"),
                            aidVal = $attachmentid.val().split(","),
                            val;
                        val = $.map(aidVal, function(n){
                            return n == param.aid ? null : n;
                        });
                        $attachmentid.val( val.join(",") );
                        $target.fadeOut(function() {
                            $target.remove();
                        });
                        $("#macro_attach_" + param.aid).remove();
                    } else {
                        Ui.tip(U.lang('DELETE_FAILED'), 'danger');
                    }
                }, 'json');
            });
        },
        'fallback': function(param, $ctx, evt) {
            // 回退上一步骤
            if ($.trim(getForm(true).content.value) === '' && Ibos.app.g('fbSigned') == 0) {
                Ui.tip('请输入回退意见', 'danger');
                $(getForm(true).content).blink();
                return false;
            }
            if (param.type == 1) {
                var form = getForm();
                var url = Ibos.app.url('workflow/handle/fallback', { key: Ibos.app.g('key') });
                $.post(url, { formhash: Ibos.app.g('formHash'), 'remind': getForm(true).content.value }, function(data) {
                    if (data.isSuccess) {
                        window.location.href = Ibos.app.url('workflow/list/index');
                    } else {
                        Ui.tip(U.lang('OPERATION_FAILED'), 'danger');
                    }
                }, 'json');
            } else {
                // 回退之前的步骤
                Ui.dialog({
                    title: WF.SELECT_FALLBACK_STEP,
                    content: document.getElementById('fallback_box'),
                    cancel: true,
                    ok: function() {
                        var selected = $('#fallback_box').find('input[type=radio]:checked').val(),
                            url = '';
                        var msg = $('#fallback_msg').val();
                        if (msg === '') {
                            Ui.tip(WF.ENTER_FALLBACK_REASON, 'danger');
                            return false;
                        }
                        if (selected) {
                            url = Ibos.app.url('workflow/handle/fallback', { key: Ibos.app.g('key'), id: selected });
                        }
                        if (url) {
                            var form = getForm();
                            form.waitingC(U.lang('IN_SUBMIT'), true);
                            $.post(url, { formhash: Ibos.app.g('formHash'), 'remind': getForm(true).content.value }, function(data) {
                                if (data.isSuccess) {
                                    window.location.href = Ibos.app.url('workflow/list/index');
                                } else {
                                    form.stopWaiting();
                                    Ui.tip(U.lang('OPERATION_FAILED'), 'danger');
                                }
                            }, 'json');
                        } else {
                            Ui.alert(WF.SELECT_FALLBACK_STEP);
                        }
                        return false;
                    }
                });
            }
            evt.preventDefault();
        }
    });
    // 判断当前浏览器是否为IE浏览器,然后处理签章手写checkbox是否可用
    if ($.browser.msie) {
        $("#IE_Check").removeAttr("disabled");
    }
    // 提示信息的初始化
    $("#support_type").tooltip({ trigger: "hover" });
    if (U.getCookie('save_flag') == 1) {
        Ui.tip(U.lang('SAVE_SUCEESS'), 'success');
        U.setCookie('save_flag', '');
    }
    if (U.getCookie('turn_flag') == 1) {
        if (Ibos.app.g('flowType') == '1') {
            showNext(false);
        } else {
            showNext(true);
        }
        U.setCookie('turn_flag', '');
    }
    $("textarea").on("change", function() {
        this.innerHTML = this.value;
    });

    var add_sign_tips = $('#add_sign_tips'),
        delayDialogInit = false;

    $('[name="addSignOrder"]').on('change', function(e) {
        if ($(this).val() === 'none') {
            add_sign_tips.text("所有人员办理完毕后，最后接收者转交下一步骤");
        } else {
            add_sign_tips.text("任何一人办理即可转交下一步骤");
        }
    });

    // 表单修改检测
    Ibos.checkFormChange('#wf_handle_form');

    /**
     * 超时设置
     */
    Ibos.evt.add({
        "timeSetting": function(param, elem) {
            var $this = $(elem),
                time = param.value,
                menu = $this.closest(".dropdown-menu"),
                parent = $this.parent(),
                text_con = $("#setting_timelimit"),
                timeunit = $("input[name=timeunit]");
            // 样式修改
            menu.children("li").removeClass("active");
            parent.addClass("active");
            // 单位值修改
            switch (time) {
                case "mins":
                    text_con.text("分");
                    break;
                case "hours":
                    text_con.text("小时");
                    break;
                case "days":
                    text_con.text("天");
                    break;
                case "weeks":
                    text_con.text("周");
                    break;
                default:
                    break;
            }
            timeunit.val(time);
        },
        "entrust": function(param, elem) {
            var opt = {
                title: Ibos.l('WF.CONFIRM_ENTRUST'),
                id: 'd_entrust',
                cancel: true,
                ok: function() {
                    $('#entrust_form').submit();
                    return false;
                }
            };

            Ui.ajaxDialog(Ibos.app.url('workflow/entrust/confirm', param), opt);
        },
        "addsign": function(param, elem) {
            var dialog = Ui.dialog({
                title: "加签",
                content: document.getElementById('addSign_box'),
                cancel: true,
                ok: function() {
                    var ids = $('#add_sign_user').val(),
                        url = Ibos.app.url('workflow/form/addsign'),
                        namesArr;

                    namesArr = $.map(ids.split(','), function() {
                        return Ibos.data.get(this);
                    });

                    $.post(url, { op: 'sign', key: param.key, behavior: $('[name="addSignOrder"]:checked').val(), uid: ids, remind: $('#add_sign_remind').val() }, function(res) {
                        if (res.isSuccess) {
                            Ui.tip("加签成功");
                            window.location.href = Ibos.app.url('workflow/list/index');
                        } else {
                            Ui.tip(res.msg, 'warning');
                            return false;
                        }
                    }, 'json');
                }
            });
        },
        "addfeedback": function(param, elem) {
            var dialog = Ui.dialog({
                title: "增加本步骤经办人",
                content: document.getElementById('addFeedBack_box'),
                cancel: true,
                ok: function() {
                    var ids = $("#add_feedback_user").val(),
                        url = Ibos.app.url('workflow/form/addsign');

                    $.post(url, { op: 'transactor', key: param.key, uid: ids, behavior: '' }, function(res) {
                        if (res.isSuccess) {
                            Ui.tip("本步骤经办人添加成功");

                            setTimeout(function(){
                                window.location.reload();
                            }, 1000);
                            return true;
                        } else {
                            Ui.tip(res.msg, 'warning');
                            return false;
                        }
                    }, 'json');
                }
            });
        },
        "delay": function(param, elem) {
            var $elem = $(elem);

            Ui.dialog({
                title: WF.HOW_LONG_IS_THE_DELAY,
                id: 'd_dialog_delay',
                init: function() {
                    if (!delayDialogInit) {
                        //初始化延期弹框中延期时间
                        $("#custom_time_datepicker").datepicker({
                            startDate: new Date
                        });

                        //点击时间选择类型中的"自定义"项后,时间选择出现
                        $(":radio[name='delay-time']", this.DOM.content).change(function() {
                            var val = $(this).val();
                            $("#custom_time_datepicker").toggleClass("show", val == "4");
                        });

                        delayDialogInit = true;
                    }
                },
                content: document.getElementById('dialog_delay'),
                cancel: true,
                ok: function() {
                    // 拿到延期类型
                    var time = this.DOM.content.find('input[name="delay-time"]:checked').val();

                    // 如果是自定义类型，则拿到自定义的具体值
                    if (time == "4") {
                        time = $("#custom_time").val();
                    }

                    $.post(Ibos.app.url('workflow/handle/adddelay'), {
                        key: param.key,
                        time: time
                    }, function(res) {
                        if (res.isSuccess) {
                            Ui.tip(WF.DELAY_OPT_SUCCESS);
                            window.location.href = Ibos.app.url('workflow/list/index');
                        } else {
                            Ui.tip(res.msg ? res.msg : Ibos.l('OPERATION_FAILED'), 'danger');
                        }
                    }, 'json');

                    return false;
                }
            });
        }
    });
});
